MODULE DaysToDate;
IMPORT IO, ConSigs;

TYPE
  TDaysInMonth = ARRAY [0..12] OF CARDINAL;

CONST
  DaysInMonth = TDaysInMonth{ 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365 };

(* Prft, ob ein Jahr ein Schaltjahr ist *)
PROCEDURE IsLeapYear(year: CARDINAL): BOOLEAN;
BEGIN
  RETURN (year MOD 400 = 0) OR ((year MOD 4 = 0) AND (year MOD 100 # 0));
END IsLeapYear;

(* Berechnet das Datum aus der Anzahl der Tage seit 1.1.0 *)
PROCEDURE DaysToDate(days: CARDINAL; VAR year, month, day: CARDINAL);
VAR
  y400, y100, y4, y1, remainingDays, i: CARDINAL;
  leapYear: BOOLEAN;
BEGIN
  (* 1. Berechnung der Jahre *)
  y400 := days DIV 146097;  (* 400-Jahres-Zyklen *)
  remainingDays := days MOD 146097;

  y100 := remainingDays DIV 36524; (* 100-Jahres-Zyklen *)
  IF y100 = 4 THEN y100 := 3; END;  (* Korrektur, wenn genau 400 Jahre *)
  remainingDays := remainingDays MOD 36524;

  y4 := remainingDays DIV 1461; (* 4-Jahres-Zyklen *)
  remainingDays := remainingDays MOD 1461;

  y1 := remainingDays DIV 365; (* Einzelne Jahre *)
  IF y1 = 4 THEN y1 := 3; END; (* Korrektur, wenn genau 4 Jahre *)
  remainingDays := remainingDays MOD 365;

  year := y400 * 400 + y100 * 100 + y4 * 4 + y1;
  leapYear := IsLeapYear(year);

  (* 2. Berechnung des Monats *)
  month := 1;
  i := 1;
  WHILE i <= 12 DO
    IF remainingDays < DaysInMonth[i] + ORD(leapYear) * ORD(i > 2) THEN
      month := i;
      i := 13; (* Beende die Schleife *)
    ELSE
      INC(i);
    END;
  END;

  (* 3. Berechnung des Tages *)
  day := remainingDays - (DaysInMonth[month - 1] + ORD(leapYear) * ORD(month > 2)) + 1;
END DaysToDate;

VAR
  days, y, m, d: CARDINAL;
BEGIN
  IO.WrStr( "DaysToDate v0, 2025-02-01, CTRL-C to exit" ); IO.WrLn();

  LOOP
    (* Eingabe: Anzahl der Tage seit 1.1.0 *)
    IO.WrStr( "Tage seit 1.1.0 eingeben: " );
    days := IO.RdCard();

    (* Berechnung des Datums *)
    DaysToDate( days, y, m, d );

    (* Ausgabe des Ergebnisses *)
    IO.WrStr( "Datum: " );
    IO.PrefixChar := '0';
    IO.WrCard( y, 4 ); IO.WrStr( "-" );
    IO.WrCard( m, 2 ); IO.WrStr( "-" );
    IO.WrCard( d, 2 ); IO.WrLn();
  END;
END DaysToDate.
